#include <fvsp.h>
#include <stdlib.h>
#include <stdbool.h>
#include <stdio.h>
#include <string.h>

const char *data[][3] = {
/*  Input,          Long output,    Short output */
    { "5.8",          "5.800.0",      "5.800" },
    { "v5.8",         "5.8.0",        "5.8" },
    { "5.002",        "5.2.0",        "5.2" },
    { "5.002",        "5.2.0",        "5.2" },
    { "5.008",        "5.8.0",        "5.8" },
    { "5.008008",     "5.8.8",        "5.8.8" },
    { "5.005",        "5.5.0",        "5.5" },
    { "5.008",        "5.8.0",        "5.8" },
    { "5.006",        "5.6.0",        "5.6" },
    { "5.006",        "5.6.0",        "5.6" },
    { "5.005",        "5.5.0",        "5.5" },
    { "5.007",        "5.7.0",        "5.7" },
    { "v5.16",        "5.16.0",       "5.16" },
    { "5.005_03",     "5.5.0",        "5.5" },
    { "5.008003",     "5.8.3",        "5.8.3" },
    { "5.008001",     "5.8.1",        "5.8.1" },
    { "5.002",        "5.2.0",        "5.2" },
    { "5.006_000",    "5.6.0",        "5.6" },
    { "5.006001",     "5.6.1",        "5.6.1" },
    { "5.006",        "5.6.0",        "5.6" },
    { "5.008.004",    "5.8.4",        "5.8.4" },
    { "5.008000",     "5.8.0",        "5.8" },
    { "1",            "1.0.0",        "1" },
    { "v1",           "1.0.0",        "1" },
    { "1.",           "1.0.0",        "1" },
    { "v1.",          "1.0.0",        "1" },
    { ".",            "0.0.0",        "" },
    { ".1",           "0.100.0",      "0.100" },
    { ".1.2",         "0.1.2",        "0.1.2" },
    { "0",            "0.0.0",        "" },
    { "._",           "0.0.0",        "" },
    { "",             "0.0.0",        "" },
    { NULL,           NULL,           NULL }
};

_Bool is(const char *got, const char *expected, const char *title) {
    static size_t counter = 1;
    _Bool passed = 0;

    if (got == NULL && expected == NULL) {
        passed = 1;
    } else if (got != NULL && expected != NULL && !strcmp(got, expected)) {
        passed = 1;
    }

    printf("%s %zd - %s\n",
            passed ? "ok" : "not ok",
            counter,
            title);
    if (!passed) {
        printf("#   Failed test '%s'\n"
               "#          got: '%s'\n"
               "#     expected: '%s'\n",
                title, got, expected);
    }

    counter++;
    return passed;
}

_Bool test(const char **test) {
    _Bool passed = 0;
    char *title = NULL;
    char *output = NULL;
    const char *input_text = test[0];

    if (input_text == NULL) input_text = "(null)";
    
    title = malloc(strlen(input_text) + 15);
    if (title == NULL) abort();

    sprintf(title, "%s %s", "long version", input_text);
    output = vsp(test[0], 0);
    passed |= is(output, test[1], title);
    free(output);

    sprintf(title, "%s %s", "short version", input_text);
    output = vsp(test[0], 1);
    passed |= is(output, test[2], title);
    free(output);

    free(title);
    return passed;
}

int main(void) {
    _Bool passed = 0;
    size_t input_count = sizeof(data)/sizeof(*data);
    
    printf("1..%zd\n", input_count * 2);

    for (size_t input = 0; input < input_count; input++) {
        passed |= test(data[input]);
    }

    exit(passed ? EXIT_SUCCESS : EXIT_FAILURE);
}
